const faye = new Faye.Client('/faye', { timeout: 120 });

// get data from socket end set UI text
faye.subscribe('/drone/navdata', data => {
  [
    'batteryPercentage',
    'clockwiseDegrees',
    'altitudeMeters',
    'frontBackDegrees',
    'leftRightDegrees',
    'xVelocity',
    'yVelocity',
    'zVelocity'
  ].forEach(type => $('#' + type).html(Math.round(data.demo[type], 4)));
  return showBatteryStatus(data.demo.batteryPercentage);
});
window.showBatteryStatus = batteryPercentage => {
  $('#batterybar').width(`${batteryPercentage}%`);
  if (batteryPercentage < 30)
    $('#batteryProgress')
      .removeClass('progress-success')
      .addClass('progress-warning');
  if (batteryPercentage < 15)
    $('#batteryProgress')
      .removeClass('progress-warning')
      .addClass('progress-danger');
  return $('#batteryProgress').attr({
    'data-original-title': `Battery status: ${batteryPercentage}%`
  });
};

// get image stream from server and set UI
faye.subscribe('/drone/image', src => $('#cam').attr({ src: src }));

// get keyinput
const keymap = {
  87: { ev: 'move', action: 'front' }, // W
  83: { ev: 'move', action: 'back' }, // S
  65: { ev: 'move', action: 'left' }, // A
  68: { ev: 'move', action: 'right' }, // D
  38: { ev: 'move', action: 'up' }, // up
  40: { ev: 'move', action: 'down' }, // down
  37: { ev: 'move', action: 'counterClockwise' }, // left
  39: { ev: 'move', action: 'clockwise' }, // right
  32: { ev: 'drone', action: 'takeoff' }, // space
  27: { ev: 'drone', action: 'land' }, // esc
  49: { ev: 'animate', action: 'flipAhead', duration: 15 }, // 1
  50: { ev: 'animate', action: 'flipLeft', duration: 15 }, // 2
  51: { ev: 'animate', action: 'yawShake', duration: 15 }, // 3
  52: { ev: 'animate', action: 'doublePhiThetaMixed', duration: 15 }, // 4
  53: { ev: 'animate', action: 'wave', duration: 15 }, // 5
  69: { ev: 'drone', action: 'disableEmergency' } // E
};
let speed = 0;
$(document).keydown(ev => {
  if (keymap[ev.keyCode] == null) return;
  ev.preventDefault();

  speed = speed >= 1 ? 1 : speed + 0.08 / (1 - speed);
  const evData = keymap[ev.keyCode];
  return faye.publish('/drone/' + evData.ev, {
    action: evData.action,
    speed: speed,
    duration: evData.duration
  });
});
$(document).keyup(ev => {
  speed = 0;
  return faye.publish('/drone/drone', { action: 'stop' });
});

// get mouse event
$('*[data-action]').on('mousedown', ev =>
  faye.publish('/drone/' + $(this).attr('data-action'), {
    action: $(this).attr('data-param'),
    speed: 0.3,
    duration: 1000 * parseInt($('#duration').val())
  })
);
$('*[data-action]').on('mouseup', ev =>
  faye.publish('/drone/move', {
    action: $(this).attr('data-param'),
    speed: $(this).attr('data-action') === 'move' ? 0 : undefined
  })
);
$('*[rel=tooltip]').tooltip();
