/*  Convert sphere coordinate system to rectangular coordinate system
 *  x = r * sin(theta) * cos(phi)
 *  y = r * sin(theta) * sin(phi)
 *  z = r * cos(theta)
 */
/*  Reverse conversion
 *  r = sqrt(x*x + y*y + z*z)
 *  theta = acos(z/r)
 *  phi = atan(y/x)
 */
/* A vector library written by 'evanw'; source: https://evanw.github.io/lightgl.js/docs/vector.html */
class Vector {
    constructor(x, y, z) {
        this.x = x || 0;
        this.y = y || 0;
        this.z = z || 0;
    }
    negative() {
      return new Vector(-this.x, -this.y, -this.z);
    }
    add(v) {
      if (v instanceof Vector) return new Vector(this.x + v.x, this.y + v.y, this.z + v.z);
      else return new Vector(this.x + v, this.y + v, this.z + v);
    }
    subtract(v) {
      if (v instanceof Vector) return new Vector(this.x - v.x, this.y - v.y, this.z - v.z);
      else return new Vector(this.x - v, this.y - v, this.z - v);
    }
    multiply (v) {
      if (v instanceof Vector) return new Vector(this.x * v.x, this.y * v.y, this.z * v.z);
      else return new Vector(this.x * v, this.y * v, this.z * v);
    }
    divide (v) {
      if (v instanceof Vector) return new Vector(this.x / v.x, this.y / v.y, this.z / v.z);
      else return new Vector(this.x / v, this.y / v, this.z / v);
    }
    equals (v) {
      return this.x == v.x && this.y == v.y && this.z == v.z;
    }
    dot (v) {
      return this.x * v.x + this.y * v.y + this.z * v.z;
    }
    cross (v) {
      return new Vector(
        this.y * v.z - this.z * v.y,
        this.z * v.x - this.x * v.z,
        this.x * v.y - this.y * v.x
      );
    }
    length () {
      return Math.sqrt(this.dot(this));
    }
    unit () {
      return this.divide(this.length());
    }
    min () {
      return Math.min(Math.min(this.x, this.y), this.z);
    }
    max () {
      return Math.max(Math.max(this.x, this.y), this.z);
    }
    toAngles () {
      return {
        theta: Math.atan2(this.z, this.x),
        phi: Math.asin(this.y / this.length())
      };
    }
    angleTo (a) {
      return Math.acos(this.dot(a) / (this.length() * a.length()));
    }
    toArray (n) {
      return [this.x, this.y, this.z].slice(0, n || 3);
    }
    clone () {
      return new Vector(this.x, this.y, this.z);
    }
    init (x, y, z) {
      this.x = x; this.y = y; this.z = z;
      return this;
    }
}

class UserInfo {
    constructor() {
        this.pos = new Vector(80, 0, 0);
        this.dist = 80;
    }
}

class DroneInfo {
    constructor() {
        this.dist = 500;
        this.pos = new Vector(500, 0, 0);  // Current position of the drone. It is changed only if a real position of dron is changed.
        this.pitch = 0;
        this.roll = 0;
        this.yow = 0;
    }
}

var origin = new Vector(0, 0, 0);

function handToDroneDelta(_nextHand) {
    var delta = _nextHand - origin;
    delta = delta.multiply()
    return posDrone.add(delta);
}